/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */


package Game;
import Game.Card.Suit;
import java.util.ArrayList;
import java.util.Random;

/**
 * Represents a pile, a collection of cards.
 * @author T.T.M. van Schaijk
 */
public class Pile extends ArrayList<Card>{

    //private ArrayList<Card> cards = new ArrayList<Card>();  // Contains the cards
    private int cardCount = 0;  // Number of cards in hand.
    private final int maxCards;

    /**
     * Should only be called by descendants.
     * @param maxCards The maximum number of cards this is allowed to hold.
     */
    public Pile(int maxCards){
        this.maxCards = maxCards;
    }

    /**
     * Default constructor sets maxCards to 0.
     */
    protected Pile(){
        this.maxCards = 0;
    }

    /**
     * Adds a card to the pile.
     * @param card Card to be added.
     */
    public void addCard(Card card){
        if(cardCount < maxCards){
            super.add(card);
            cardCount++;
        }
    }

    public void removeCard(int index){
        if(index >= 0 && index<cardCount){
            super.remove(index);
            cardCount--;
        }
    }

    public void removeCard(Card card) {
        for (int c=0; c<cardCount; c++){
            if (this.getCard(c).getRank() == card.getRank() &&
                    this.getCard(c).getSuit() == card.getSuit()) {
                super.remove(c);
                cardCount--;
                return;
            }
        }
        System.err.println("ERROR: Card cannot be removed as it is not in hand.");
    }

    /**
     * Shuffles the pile, meaning the pile will be a random permutation.
     */
    public void shuffle(){
        ArrayList<Card> tmpCards = new ArrayList<Card>();

        Random randomGenerator = new Random();

        for(int i=cardCount; i>0; i--) {
            int rnd = randomGenerator.nextInt(i);
            tmpCards.add(super.get(rnd));
            super.remove(rnd);
        }

        super.clear();
        super.addAll(tmpCards);
    }

    /**
     * Returns and removes the top card.
     * @return Top card of this pile.
     */
    public Card getCard() {
        if(cardCount>0) {
            Card ret = super.get(0);
            super.remove(0);
            cardCount--;
            return ret;
        } else {
            return null;
        }
    }

    /**
     * Return the requestedCard without removing it.
     * @param requestedCard The number of the requested card.
     * @return The requested card.
     */
    public Card getCard(int requestedCard) {
        if(cardCount > requestedCard) {
            return super.get(requestedCard);
        } else {
            return null;
        }
    }

    /**
     * Clears the contents of this pile.
     */
    public void empty() {
        super.clear();
        cardCount = 0;
    }

    /**
     * Returns the number of cards in the pile.
     * @return Number of cards in this pile.
     */
    public int getCardCount() {
        return cardCount;
    }

    public Pile getCardBySuit(Suit suit) {  //returns Pile of all cards that matches suit
        Pile temp = new Pile(maxCards);

        for (int i=0; i<getCardCount(); i++){
            if (getCard(i).getSuit() == suit){
                temp.addCard(getCard(i));
            }
        }

        return temp;
    }

    public Pile getCardBySuitExcept(Suit suit) {        //returns Pile of all cards except for cards with suit == suit
        Pile temp = new Pile(maxCards);

        for (int i=0; i<getCardCount(); i++) {
            if (getCard(i).getSuit() != suit) {
                temp.addCard(getCard(i));
            }
        }

        return temp;
    }

    public Pile getAllCards() {         //returns Pile of all cards still in hand
        Pile temp = new Pile(maxCards);

        for (int i=0; i<getCardCount(); i++) {
            temp.addCard(getCard(i));
        }

        return temp;
    }

    public boolean hasSuit(Suit suit) {        //checks if Hand has specific suit
        for (int i=0; i < this.getCardCount() ; i++) {
            if (super.get(i).getSuit() == suit) {
                return true;
            }
        }
        return false;
    }

    public void sort(Card.Suit trump) {
        ArrayList<Card> tmpCards = new ArrayList<Card>();

        for(int i=cardCount; i>0; i--) {
            int lowestN = 0;
            Card lowest = super.get(0);
            for(int j=1; j<i; j++) {
                Card card = super.get(j);
                if(lowest==null || lowest.higherOrder(card, trump)) {
                    lowest = card;
                    lowestN = j;
                }
            }
            tmpCards.add(lowest);
            super.remove(lowestN);
        }

        super.clear();
        super.addAll(tmpCards);
    }
}
